package main

import (
	"encoding/json"
	"errors"
	"fmt"
	"log"
	"net/http"
	"strings"
)

type DispatchFunction func([]string, []byte) ([]byte, error)

var apiRouting = map[string]*(map[string]DispatchFunction){
	"version":   &ApiVersionRouting,
	"purchases": &ApiPurchasesRouting,
	"products":  &ApiProductsRouting,
	"needs":     &ApiNeedsRouting,
}

func ApiRouting(w http.ResponseWriter, r *http.Request) {
	log.Printf("Handling request %s: %s\n", r.Method, r.URL.Path)

	// Extract URL arguments
	var sURL = strings.Split(r.URL.Path, "/")
	if sURL[len(sURL)-1] == "" && len(sURL) > 2 {
		// Remove trailing /
		sURL = sURL[:len(sURL)-1]
	}

	var ret []byte
	var err error = nil

	// Read the body
	if r.ContentLength < 0 || r.ContentLength > 65536 {
		http.Error(w, fmt.Sprintf("{errmsg:\"Request too large or request size unknown\"}", err), http.StatusRequestEntityTooLarge)
		return
	}
	var body []byte
	if r.ContentLength > 0 {
		tmp := make([]byte, 1024)
		for {
			n, err := r.Body.Read(tmp)
			for j := 0; j < n; j++ {
				body = append(body, tmp[j])
			}
			if err != nil || n <= 0 {
				break
			}
		}
	}

	// Route request
	if len(sURL) > 2 {
		if h, ok := apiRouting[sURL[2]]; ok {
			if f, ok := (*h)[r.Method]; ok {
				ret, err = f(sURL[3:], body)
			} else {
				err = errors.New(fmt.Sprintf("Invalid action (%s) provided for %s.", r.Method, sURL[2]))
			}
		}
	} else {
		err = errors.New(fmt.Sprintf("No action provided for %s", sURL[1]))
	}

	// Format response
	if err == nil && len(ret) > 0 {
		w.Write(ret)
	} else if err != nil {
		if j, err := json.Marshal(map[string]string{"errmsg": err.Error()}); err == nil {
			http.Error(w, string(j), http.StatusBadRequest)
		} else {
			http.Error(w, fmt.Sprintf("{errmsg:\"%q\"}", err), http.StatusInternalServerError)
		}
	} else if j, err := json.Marshal(map[string]string{"errmsg": "Page not found"}); err != nil {
		http.Error(w, fmt.Sprintf("{errmsg:\"%q\"}", err), http.StatusInternalServerError)
	} else {
		http.Error(w, string(j), http.StatusNotFound)
	}
}
